/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gio/gio.h>

#include "main.h"
#include "image-utils.h"

void
gtk_image_set_black (GtkWidget *img, WkView *view)
{
    int w, h;
    GdkPixbuf *pxb;

    gdk_drawable_get_size (GTK_WIDGET (view)->window, &w, &h);

    pxb = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
    gdk_pixbuf_fill (pxb, 0);

    gtk_image_set_from_pixbuf (GTK_IMAGE (img), pxb);

    g_object_unref (pxb);
}

GdkPixbuf *
get_pixbuf_from_filename (const gchar *filename, WkView *view)
{
    gint w, h, iw, ih;
    gint dx, dy, xo, yo;
    gdouble xscale, yscale;
    GError *error = NULL;
    GdkPixbuf *pxb, *final = NULL;

    g_return_val_if_fail (filename != NULL, NULL);

    /* get the final size of the image */
    gdk_drawable_get_size (GTK_WIDGET (view)->window, &w, &h);

    pxb = gdk_pixbuf_new_from_file (filename, &error);
    if (!pxb)
    {
        DEBUG ("Image could not be loaded: %s", error->message);
        g_error_free (error);
        return NULL;
    }

    iw = gdk_pixbuf_get_width (pxb);
    ih = gdk_pixbuf_get_height (pxb);

    /* black background */
    final = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
    gdk_pixbuf_fill (final, 0);

    if ((iw > w) || (ih > h))
    {
        /* scale the image if it's larger than the WkView */
        xscale = (gdouble) w / iw;
        yscale = (gdouble) h / ih;
        if (xscale < yscale) {
            yscale = xscale;
            xo = 0;
            yo = (h - (ih * yscale)) / 2;
        } else {
            xscale = yscale;
            xo = (w - (iw * xscale)) / 2;
            yo = 0;
        }
        dx = xo;
        dy = yo;

        gdk_pixbuf_composite (pxb, final, dx, dy,
            iw * xscale, ih * yscale, xo, yo, xscale, yscale,
            GDK_INTERP_BILINEAR, 255);
    }
    else
    {
        /* center the image if it's smaller than the WkView */
        dx = MAX((w - iw) / 2, 0);
        dy = MAX((h - ih) / 2, 0);
        xo = MIN((w - iw) / 2, dx);
        yo = MIN((h - ih) / 2, dy);

        gdk_pixbuf_composite (pxb, final, dx, dy,
            MIN (w, iw), MIN (h, ih), xo, yo, 1.0, 1.0,
            GDK_INTERP_NEAREST, 255);
    }


    g_object_unref (pxb);

    return final;
}
