/**
 * Copyright (C) 2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "main.h"
#include "wk-events.h"
#include "wk-browser.h"
#include "wk-daemon.h"

/* in ms */
#define EVENT_TIMEOUT 500
#define NOTIFY_EVENT_TIMEOUT 100

static guint32 last_event = 0;
static int timeout_id = 0;

gboolean
on_timeout (gpointer data)
{
    WkBrowser *browser = WK_BROWSER (wk_browser_get (NULL));
    WkDaemon *d = wk_daemon_get ();

    wk_browser_clear_session (browser, TRUE);
    wk_daemon_handle_stop_session (d);

    return FALSE;
}

void
event_no_filter (GdkEvent *ev, gpointer data)
{
    GSource *source;
    static WkSettings *settings = NULL;

    if ((ev->type < GDK_MOTION_NOTIFY) || (ev->type > GDK_FOCUS_CHANGE))
    {
        gtk_main_do_event (ev);
        return;
    }

    if (! settings)
        settings = GET_SETTINGS ();

    if (!settings->stop_session_after_seconds)
    {
        gtk_main_do_event (ev);
        return;
    }

    if (timeout_id &&
        (source = g_main_context_find_source_by_id (NULL, timeout_id)) != NULL)
        g_source_destroy (source);

    timeout_id = g_timeout_add_seconds (settings->stop_session_after_seconds,
                                        on_timeout, NULL);

    last_event = gdk_event_get_time (ev);
    gtk_main_do_event (ev);
}

void
event_filter (GdkEvent *ev, gpointer data)
{
    static WkDaemon *d = NULL;
    static WkSettings *settings = NULL;
    static guint32 last_notify_event = 0;
    guint32 t;

    if (ev->type == GDK_KEY_PRESS)
    {
        GdkEventKey *ek = (GdkEventKey *)ev;
        if ((((ek->state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
              && (ek->keyval == GDK_q))
            || (ek->keyval == GDK_Escape))
        {
            gtk_main_do_event (ev);
            return;
        }
    }

    t = gdk_event_get_time (ev);

    if ((ev->type >= GDK_MOTION_NOTIFY)
        && (ev->type <= GDK_LEAVE_NOTIFY)
        && (t - last_event > EVENT_TIMEOUT))
    {
        /*
         * When we show a new slideshow item, we might change the tab of the
         * notebook, generating 2 events (leave and enter). We add a timeout
         * to handle such situations.
         */
        if ((ev->type == GDK_LEAVE_NOTIFY) || (ev->type == GDK_ENTER_NOTIFY))
        {
            if (!last_notify_event || ((t - last_notify_event) <= NOTIFY_EVENT_TIMEOUT))
            {
                last_notify_event = t;
                return;
            }

            last_notify_event = t;
        }

        if (!settings)
            settings = GET_SETTINGS ();

        if ((settings->mode == MODE_KIOSK) && settings->slideshow_stop_on_event)
        {
            last_notify_event = 0;

            if (!d)
                d = wk_daemon_get ();
            wk_daemon_handle_start_session (d);
        }

        return;
    }

    gtk_main_do_event (ev);
}

void
wk_event_set_event_filter ()
{
    GSource *source;

    DEBUG ("setting up event filter");

    if (timeout_id &&
        (source = g_main_context_find_source_by_id (NULL, timeout_id)) != NULL)
    {
        g_source_destroy (source);
    }

    gdk_event_handler_set (event_filter, NULL, NULL);
}


