/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WK_VIEW_H
#define WK_VIEW_H

#include <gtk/gtk.h>
#include <webkit/webkit.h>

G_BEGIN_DECLS

typedef enum
{
    WK_VIEW_STATE_NONE,
    WK_VIEW_STATE_DISPLAY,
    WK_VIEW_STATE_WEB
} WkViewState;

typedef struct _WkViewPrivate WkViewPrivate;
typedef struct _WkViewClass   WkViewClass;
typedef struct _WkView        WkView;

#define WK_TYPE_VIEW             (wk_view_get_type ())
#define WK_VIEW(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), WK_TYPE_VIEW, WkView))
#define WK_VIEW_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), WK_TYPE_VIEW, WkViewClass))
#define WK_IS_VIEW(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WK_TYPE_VIEW))
#define WK_IS_VIEW_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), WK_TYPE_VIEW))
#define WK_VIEW_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), WK_TYPE_VIEW, WkViewClass))

struct _WkViewClass
{
    GtkNotebookClass __parent__;

    void (*session_start)   (WkView *view);
    void (*session_end)     (WkView *view);
    void (*time_added)      (WkView *view);
};

struct _WkView
{
    GtkNotebook    __parent__;

    WebKitWebView *webview;
    GtkWidget     *image;
    GtkWidget     *video;

    WkViewPrivate *priv;
};

GType wk_view_get_type (void) G_GNUC_CONST G_GNUC_INTERNAL;

GtkWidget *wk_view_new (void);

WkViewState wk_view_get_state (WkView *view);

void wk_view_set_url      (WkView *view,
                           gchar  *url);
void wk_view_set_playlist (WkView *view,
                           gchar  *playlist);

void wk_view_show_toolbar (WkView  *view,
                           gboolean show);

guint wk_view_get_time (WkView *view);
void  wk_view_add_time (WkView *view,
                        guint   seconds);

void wk_view_start_session (WkView *view);
void wk_view_stop_session  (WkView *view);

gboolean wk_view_show (WkView *view, guint type, const gchar *uri);

void wk_view_clear_history (WkView *view);

void wk_view_start (WkView *view);

void wk_view_url_entry_grab_focus (WkView *view);

G_END_DECLS

#endif /* !WK_VIEW_H */
