/**
 * Copyright (C) 2009-2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#define PROGNAME "lwk-admin"
#define DBUSBASE "dbus-send --system /com/linutop/WebKiosk " \
                 " com.linutop.WebKiosk.Signal."

void
usage ()
{
    printf (PROGNAME ": manage Linutop Webkiosk sessions\n");
    printf ("\n");
    printf ("Usage:\n");
    printf ("  lwk-admin start|stop|quit\n");
    printf ("\n");
    printf ("Options:\n");
    printf ("  -h, --help      Display this help\n");
    printf ("  -v, --version   Display version informations\n");
    printf ("\n");
}

int
main (int argc, char **argv)
{
    if (argc == 1)
    {
        usage ();
        return 1;
    }

    if (strcmp (argv[1], "--help") == 0 || strcmp (argv[1], "-h") == 0)
    {
        usage ();
        return 0;
    }
    else if (strcmp (argv[1], "--version") == 0 || strcmp (argv[1], "-v") == 0)
    {
        printf ("%s %s\n", PROGNAME, PACKAGE_VERSION);
        return 0;
    }

    if (setuid (0) != 0)
    {
        fprintf (stderr,
                 "Unable to setuid root; you probably don't "
                 "have the permission to use this tool\n");
        return 2;
    }

    if (strcmp (argv[1], "start") == 0)
        return system (DBUSBASE "StartSession");
    else if (strcmp (argv[1], "stop") == 0)
        return system (DBUSBASE "StopSession");
    else if (strcmp (argv[1], "quit") == 0)
        return system (DBUSBASE "Quit");

    else
    {
        usage ();
        return 1;
    }

    return 0;
}

